@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2020 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -442,8 +442,7 @@ prefix_section_parser (enum mu_cfg_section_stage stage,
 	}
       else if (!pfx->prefix)
 	{
-	  pfx->prefix = mu_alloc (strlen (pfx->dir) + 1);
-	  translate_delim (pfx->prefix, pfx->dir, pfx->delim, '/');
+	  pfx->prefix = namespace_encode_delim (pfx, pfx->dir);
 	}
     }
   return 0;
@@ -811,8 +810,6 @@ imap4d_mainloop (int ifd, int ofd, struct imap4d_srv_config *cfg)
   tokp = imap4d_tokbuf_init ();
   while (1)
     {
-      if (idle_timeout && io_wait_input (idle_timeout) != 1)
-	imap4d_bye (ERR_TIMEOUT);
       imap4d_readline (tokp);
       /* check for updates */
       imap4d_sync ();
