@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2020 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,12 +18,50 @@
 #include <dirent.h>
 #include <pwd.h>
 
+/*
+ * IMPORTANT NOTE:
+ *
+ * The LIST command takes two arguments: 'reference' and 'mailbox',
+ * which in the code below is referred to as 'wcard'.  Now, RFC 3501
+ * states, that:
+ *
+ *  If the reference argument is not a level of mailbox
+ *  hierarchy (that is, it is a \NoInferiors name), and/or
+ *  the reference argument does not end with the hierarchy
+ *  delimiter, it is implementation-dependent how this is
+ *  interpreted.  For example, a reference of "foo/bar" and
+ *  mailbox name of "rag/baz" could be interpreted as
+ *  "foo/bar/rag/baz", "foo/barrag/baz", or "foo/rag/baz".
+ *  A client SHOULD NOT use such a reference argument except
+ *  at the explicit request of the user.  A hierarchical
+ *  browser MUST NOT make any assumptions about server
+ *  interpretation of the reference unless the reference is
+ *  a level of mailbox hierarchy AND ends with the hierarchy
+ *  delimiter.
+ *
+ * Mailutils' approach is basically to concatenate the two
+ * arguments with a hierarchy separator (as per RFC 2342) in
+ * between.  In detail:
+ *
+ *  1. Given two arguments, 'reference' and 'wcard', the reference is
+ *     used to look up the matching namespace (first approximation).
+ *  2. If wcard contains non-wildcard directory prefix, that prefix
+ *     is removed and appended to the reference, separated by the
+ *     namespace delimiter.
+ *  3. The updated reference is used to look up the final namespace.
+ *  4. If the namespace prefix ends with a delimiter, wcard is appended
+ *     to it.
+ *  5. Otherwise, the name to look up is formed by concatenating the
+ *     namespace prefix, namespace delimiter, and wcard.
+ */
+
 struct refinfo
 {
   char const *refptr;   /* Original reference */
   size_t reflen;        /* Length of the original reference */
   struct namespace_prefix const *pfx;
-  size_t dirlen;        /* Length of the current directory prefix */
+  int delim;            /* If not 0, this character will be inserted between
+			   the original reference and mailbox name */
   char *buf;
   size_t bufsize;
 };
@@ -36,7 +74,7 @@ list_fun (mu_folder_t folder, struct mu_list_response *resp, void *data)
   size_t size;
   char *p;
 
-  name = resp->name + refinfo->dirlen;
+  name = resp->name;
 
   /* There can be only one INBOX */
   if (refinfo->reflen == 0 &&  mu_c_strcasecmp (name, "INBOX") == 0)
@@ -77,12 +115,14 @@ list_fun (mu_folder_t folder, struct mu_list_response *resp, void *data)
     {
       memcpy (refinfo->buf, refinfo->refptr, refinfo->reflen);
       p = refinfo->buf + refinfo->reflen;
+      if (refinfo->delim)
+	*p++ = refinfo->delim;
     }
   else
     p = refinfo->buf;
   if (*name)
     translate_delim (p, name, refinfo->pfx->delim, resp->separator);
-
+  
   name = refinfo->buf;
   
   if (strpbrk (name, "\"{}"))
@@ -97,6 +137,7 @@ list_fun (mu_folder_t folder, struct mu_list_response *resp, void *data)
   return 0;
 }
 
+/* Return 1 if the string REF matches exactly the prefix in PFX. */
 static int
 match_pfx (struct namespace_prefix const *pfx, char const *ref)
 {
@@ -104,7 +145,9 @@ match_pfx (struct namespace_prefix const *pfx, char const *ref)
 
   for (; *q; p++, q++)
     {
-      if (*p == 0 || *p != *q)
+      if (*p == 0)
+	return *q == pfx->delim && q[1] == 0;
+      if (*p != *q)
 	return 0;
     }
   if (*p == pfx->delim)
@@ -119,8 +162,6 @@ list_ref (char const *ref, char const *wcard, char const *cwd,
   int rc;
   struct refinfo refinfo;
   mu_folder_t folder;
-  char const *dir;
-  mu_url_t url;
   struct mu_folder_scanner scn = MU_FOLDER_SCANNER_INITIALIZER;
 
   if (!wcard[0])
@@ -168,14 +209,10 @@ list_ref (char const *ref, char const *wcard, char const *cwd,
   refinfo.refptr = ref;
   refinfo.reflen = strlen (ref);
 
-  mu_folder_get_url (folder, &url);
-  mu_url_sget_path (url, &dir);
-  refinfo.dirlen = strlen (dir);
-
-  if (refinfo.refptr[refinfo.reflen-1] == pfx->delim)
-    refinfo.reflen--;
-  else if (strcmp (ref, pfx->prefix) == 0)
-    refinfo.dirlen++;
+  /* Insert delimiter after the reference prefix, unless the latter already
+     ends with a delimiter or is the same as the namespace prefix. */
+  if (ref[refinfo.reflen-1] != pfx->delim && strcmp (ref, pfx->prefix))
+    refinfo.delim = pfx->delim;
   
   /* The special name INBOX is included in the output from LIST, if
      INBOX is supported by this server for this user and if the
@@ -189,7 +226,7 @@ list_ref (char const *ref, char const *wcard, char const *cwd,
       mu_imap_wildmatch_ci (wcard, "INBOX", MU_HIERARCHY_DELIMITER) == 0)
     io_untagged_response (RESP_NONE, "LIST (\\NoInferiors) NIL INBOX");
 
-  scn.pattern = (void*) wcard;
+  scn.pattern = namespace_decode_delim (pfx, wcard);
   scn.enumfun = list_fun;
   scn.enumdata = &refinfo;
   if (refinfo.pfx->record)
@@ -199,6 +236,7 @@ list_ref (char const *ref, char const *wcard, char const *cwd,
     }
   
   mu_folder_scan (folder, &scn);
+  free (scn.pattern);
   mu_list_destroy (&scn.records);
   mu_folder_destroy (&folder);
   free (refinfo.buf);
