@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2019 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -63,6 +63,9 @@ struct mu_address
   struct mu_address *next;
 };
 
+#define MU_ADDRESS_HINT_INITIALIZER \
+  { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL }
+  
 extern int mu_address_create_null (mu_address_t *);
   
 extern int mu_address_create_hint (mu_address_t *, const char *,
@@ -146,16 +149,6 @@ extern int mu_address_union (mu_address_t *a, mu_address_t b);
   
 extern int mu_stream_format_address (mu_stream_t str, mu_address_t addr);
 
-  /* Deprecated calls */
-  
-extern int mu_address_to_string (mu_address_t, char *, size_t, size_t *)
-  MU_DEPRECATED; /* Use mu_address_get_printable, if you really have to */
-  
-extern size_t mu_address_format_string (mu_address_t addr, char *buf,
-					size_t buflen)
-  MU_DEPRECATED; /* Use mu_stream_format_address, or any of the
-		    _get_printable functions */
-
 extern int mu_validate_email (mu_address_t subaddr);
 
   
