@@ -1,5 +1,5 @@
 /* cfg.h -- general-purpose configuration file parser 
-   Copyright (C) 2007-2012, 2014-2017 Free Software Foundation, Inc.
+   Copyright (C) 2007-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -23,6 +23,7 @@
 #include <mailutils/debug.h>
 #include <mailutils/opool.h>
 #include <mailutils/util.h>
+#include <mailutils/locus.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -64,7 +65,7 @@ enum mu_cfg_node_type
 
 struct mu_cfg_node
 {
-  struct mu_locus locus;
+  struct mu_locus_range locus;
   enum mu_cfg_node_type type;
   char *tag;
   mu_config_value_t *label;
@@ -82,31 +83,34 @@ struct mu_cfg_parse_hints
 
 /* Bit constants for the flags field of struct mu_cfg_parse_hints */
 /* Parse site-wide configuration file hints.site_file */
-#define MU_CFHINT_SITE_FILE        0x0001
+#define MU_CFHINT_SITE_FILE          0x0001
 /* Parse custom configuration file hints.custom_file */
-#define MU_CFHINT_CUSTOM_FILE      0x0002
+#define MU_CFHINT_CUSTOM_FILE        0x0002
 /* The hints.program field is set.  The "program PROGNAME" section
    will be processed, if PROGNAME is the same as hints.program.
    If include statement is used with the directory name DIR as its
    argument, the file DIR/PROGNAME will be looked up and read in,
    if it exists. */
-#define MU_CFHINT_PROGRAM          0x0004
+#define MU_CFHINT_PROGRAM            0x0004
 
 /* If MU_CFHINT_PROGRAM is set, look for the file ~/.PROGNAME after parsing
    site-wide configuration */
-#define MU_CFHINT_PER_USER_FILE    0x0008
-  
+#define MU_CFHINT_PER_USER_FILE      0x0008
+
+/* Don't allow to override configuration settings from the command line. */
+#define MU_CFHINT_NO_CONFIG_OVERRIDE 0x0010
+	
 /* Verbosely log files being processed */
-#define MU_CF_VERBOSE              0x0010
+#define MU_CF_VERBOSE                0x0100
 /* Dump the pare tree on stderr */
-#define MU_CF_DUMP                 0x0020
+#define MU_CF_DUMP                   0x0200
 
 /* Format location of the statement */
-#define MU_CF_FMT_LOCUS            0x0100
+#define MU_CF_FMT_LOCUS              0x1000
 /* Print only value */
-#define MU_CF_FMT_VALUE_ONLY       0x0200
+#define MU_CF_FMT_VALUE_ONLY         0x2000
 /* Print full parameter path */
-#define MU_CF_FMT_PARAM_PATH       0x0400
+#define MU_CF_FMT_PARAM_PATH         0x4000
   
 struct mu_cfg_tree
 {
@@ -119,8 +123,6 @@ int mu_cfg_tree_union (mu_cfg_tree_t **pa, mu_cfg_tree_t **pb);
 int mu_cfg_tree_postprocess (mu_cfg_tree_t *tree,
 			     struct mu_cfg_parse_hints *hints);
 
-extern struct mu_locus mu_cfg_locus;
-
 mu_opool_t mu_cfg_lexer_pool (void);
 
 #define MU_CFG_ITER_OK   0
@@ -180,13 +182,18 @@ typedef int (*mu_cfg_section_fp) (enum mu_cfg_section_stage stage,
 
 struct mu_cfg_section
 {
-  const char *ident;
-  char *label;
-  mu_cfg_section_fp parser;
-  void *target;
-  size_t offset;
+  const char *ident;              /* Section identifier */  
+  char *label;                    /* Label description */
+  mu_cfg_section_fp parser;       /* Parser function */
+  void *data;                     /* Data pointer */
+  size_t offset;                  /* Offset within target (see below) */
   mu_list_t /* of mu_cfg_cont */ children;
-  char *docstring;
+  char *docstring;                /* Documentation string */
+  void *target;                   /* Actual pointer to the data. It is
+				     recomputed each time the section is
+				     reduced. */
+  char *ident_storage;            /* Storage for ident, if malloc'ed */
+  char *label_storage;            /* Same for the label */
 };
 
 enum mu_cfg_cont_type
@@ -282,7 +289,7 @@ int mu_cfg_parse_file (mu_cfg_tree_t **return_tree, const char *file,
 int mu_cfg_tree_create (struct mu_cfg_tree **ptree);
 mu_cfg_node_t *mu_cfg_tree_create_node (struct mu_cfg_tree *tree,
 					enum mu_cfg_node_type type,
-					const struct mu_locus *loc,
+					const struct mu_locus_range *loc,
 					const char *tag,
 					const char *label,
 					mu_list_t nodelist);
