@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2020 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -31,6 +31,7 @@
 #include <stdio.h>
 #include <time.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <errno.h>
@@ -87,7 +88,8 @@ static int amd_message_unseen (mu_mailbox_t, size_t *);
 static int amd_expunge (mu_mailbox_t);
 static int amd_sync (mu_mailbox_t);
 static int amd_uidnext (mu_mailbox_t mailbox, size_t *puidnext);
-static int amd_uidvalidity (mu_mailbox_t, unsigned long *);
+static int amd_get_uidvalidity (mu_mailbox_t, unsigned long *);
+static int amd_set_uidvalidity (mu_mailbox_t, unsigned long);
 static int amd_scan (mu_mailbox_t, size_t, size_t *);
 static int amd_is_updated (mu_mailbox_t);
 static int amd_get_size (mu_mailbox_t, mu_off_t *);
@@ -223,6 +225,12 @@ _amd_prop_create (struct _amd_data *amd)
       free (mhprop);
       return errno;
     }
+
+  if (access (mhprop->filename, F_OK) == 0)
+    {
+      amd->flags |= MU_AMD_F_PROP;
+    }
+  
   rc = mu_property_create_init (&amd->prop, mu_mh_property_init, mhprop);
   if (rc)
     {
@@ -254,18 +262,22 @@ amd_msg_bsearch (struct _amd_data *amd, mu_off_t first, mu_off_t last,
   mu_off_t mid;
   int rc;
 
-  if (last < first)
-    return 1;
-  
-  mid = (first + last) / 2;
-  rc = amd->msg_cmp (amd->msg_array[mid], msg);
-  if (rc > 0)
-    return amd_msg_bsearch (amd, first, mid-1, msg, pret);
-  *pret = mid;
-  if (rc < 0)
-    return amd_msg_bsearch (amd, mid+1, last, msg, pret);
-  /* else */
-  return 0;
+  while (first <= last)
+    {
+      mid = (first + last) / 2;
+      rc = amd->msg_cmp (amd->msg_array[mid], msg);
+      if (rc > 0)
+	last = mid - 1;
+      else
+	{
+	  *pret = mid;
+	  if (rc < 0)
+	    first = mid + 1;
+	  else
+	    return 0;
+	}
+    }
+  return 1;
 }
 
 /* Search for message MSG in the message array of AMD.
@@ -318,8 +330,7 @@ amd_msg_lookup (struct _amd_data *amd, struct _amd_message *msg,
     }
   
   rc = amd_msg_bsearch (amd, 0, amd->msg_count - 1, msg, &i);
-  if (rc == 0)
-    *pret = i + 1;
+  *pret = i + 1;
   return rc;
 }
 
@@ -408,7 +419,8 @@ amd_init_mailbox (mu_mailbox_t mailbox, size_t amd_size,
   mailbox->_message_unseen = amd_message_unseen;
   mailbox->_expunge = amd_expunge;
   mailbox->_sync = amd_sync;
-  mailbox->_uidvalidity = amd_uidvalidity;
+  mailbox->_get_uidvalidity = amd_get_uidvalidity;
+  mailbox->_set_uidvalidity = amd_set_uidvalidity;
   mailbox->_uidnext = amd_uidnext;
 
   mailbox->_scan = amd_scan;
@@ -488,7 +500,7 @@ amd_open (mu_mailbox_t mailbox, int flags)
   _amd_prop_create (amd);
   
   if (mailbox->locker == NULL)
-    mu_locker_create (&mailbox->locker, "/dev/null", 0);
+    mu_locker_create_ext (&mailbox->locker, "/dev/null", NULL);
   
   return 0;
 }
@@ -532,7 +544,7 @@ amd_message_qid (mu_message_t msg, mu_message_qid_t *pqid)
 {
   struct _amd_message *mhm = mu_message_get_owner (msg);
   
-  return mhm->amd->cur_msg_file_name (mhm, pqid);
+  return mhm->amd->cur_msg_file_name (mhm, 0, pqid);
 }
 
 static void
@@ -800,7 +812,7 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm,
     {
       /* Unlink the original file */
       char *old_name;
-      status = amd->cur_msg_file_name (mhm, &old_name);
+      status = amd->cur_msg_file_name (mhm, 1, &old_name);
       free (msg_name);
       if (status == 0 && unlink (old_name))
 	status = errno;
@@ -851,8 +863,6 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm,
 	break;
 
       if (!(mu_c_strncasecmp (buf, "status:", 7) == 0
-	    || mu_c_strncasecmp (buf, "x-imapbase:", 11) == 0
-	    || mu_c_strncasecmp (buf, "x-uid:", 6) == 0
 	    || mu_c_strncasecmp (buf, 
                 MU_HEADER_ENV_DATE ":", sizeof (MU_HEADER_ENV_DATE)) == 0
 	    || mu_c_strncasecmp (buf, 
@@ -938,7 +948,7 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm,
   free (buf);
   fclose (fp);
 
-  status = amd->cur_msg_file_name (mhm, &old_name);
+  status = amd->cur_msg_file_name (mhm, 1, &old_name);
   if (status == 0)
     {
       if (rename (name, msg_name))
@@ -977,8 +987,6 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm,
 	    unlink (old_name);
 	}
       free (old_name);
-      
-      mhm->orig_flags = mhm->attr_flags;
     }
   free (msg_name);
   free (name);
@@ -1047,7 +1055,7 @@ amd_append_message (mu_mailbox_t mailbox, mu_message_t msg)
   if (status == 0 && mailbox->observable)
     {
       char *qid;
-      if (amd->cur_msg_file_name (mhm, &qid) == 0)
+      if (amd->cur_msg_file_name (mhm, 0, &qid) == 0)
 	{
 	  mu_observable_notify (mailbox->observable, 
 	                        MU_EVT_MAILBOX_MESSAGE_APPEND,
@@ -1333,7 +1341,7 @@ amd_expunge (mu_mailbox_t mailbox)
 	      char *old_name;
 	      char *new_name;
 
-	      rc = amd->cur_msg_file_name (mhm, &old_name);
+	      rc = amd->cur_msg_file_name (mhm, 1, &old_name);
 	      if (rc)
 		return rc;
 	      rc = amd->new_msg_file_name (mhm, mhm->attr_flags, 1,
@@ -1378,6 +1386,7 @@ amd_expunge (mu_mailbox_t mailbox)
 	  pp = amd_pool_lookup (mhm);
 	  if (pp)
 	    *pp = NULL;
+	  amd_message_stream_close (mhm);
 	  mu_message_destroy (&mhm->message, mhm);
 	  if (amd->msg_free)
 	    amd->msg_free (mhm);
@@ -1402,7 +1411,16 @@ amd_expunge (mu_mailbox_t mailbox)
 
   if (expcount)
     {
+      int reset_uidvalidity;
+      
       last_expunged++;
+
+      /* See the description of MU_AMD_VOLATILE_UIDNEXT in amd.h for
+	 details.
+      */
+      reset_uidvalidity = (amd->capabilities & MU_AMD_VOLATILE_UIDNEXT) 
+	                   && last_expunged == amd->msg_count;
+      
       do
 	{
 	  size_t j;
@@ -1417,6 +1435,18 @@ amd_expunge (mu_mailbox_t mailbox)
 	    ;
 	}
       while (last_expunged);
+
+      if (reset_uidvalidity)
+	{
+      	  /*
+	   * The following is equivalent to
+	   * mu_mailbox_uidvalidity_reset (amd->mailbox);
+	   */
+
+	  struct timeval tv;
+	  gettimeofday (&tv, NULL);
+	  amd_set_uidvalidity (amd->mailbox, tv.tv_sec);
+	}
     }
   
   if (updated && !amd->mailbox_size)
@@ -1463,45 +1493,63 @@ amd_sync (mu_mailbox_t mailbox)
   return 0;
 }
 
+static inline int
+amd_initial_scan (struct _amd_data *amd)
+{
+  if (!(amd->flags & MU_AMD_F_INIT_SCAN))
+    {
+      int status = _amd_scan0 (amd, 1, NULL, 0);
+      if (status != 0)
+        return status;
+      amd->flags |= MU_AMD_F_INIT_SCAN;
+    }
+  return 0;
+}
+
 static int
-amd_uidvalidity (mu_mailbox_t mailbox, unsigned long *puidvalidity)
+amd_get_uidvalidity (mu_mailbox_t mailbox, unsigned long *pval)
 {
   struct _amd_data *amd = mailbox->data;
-  int status = amd_messages_count (mailbox, NULL);
+  int status = amd_initial_scan (amd);
   if (status != 0)
     return status;
-  /* If we did not start scanning yet do it now.  */
-  if (amd->msg_count == 0)
-    {
-      status = _amd_scan0 (amd, 1, NULL, 0);
-      if (status != 0)
-	return status;
-    }
+  return _amd_prop_fetch_ulong (amd, _MU_AMD_PROP_UIDVALIDITY, pval);
+}
 
-  return _amd_prop_fetch_ulong (amd, _MU_AMD_PROP_UIDVALIDITY, puidvalidity);
+static int
+amd_set_uidvalidity (mu_mailbox_t mailbox, unsigned long uidvalidity)
+{
+  struct _amd_data *amd = mailbox->data;
+  size_t uidnext;
+  int status = amd_initial_scan (amd);
+  if (status != 0)
+    return status;
+
+  if (amd->msg_count == 0)
+    uidnext = 1;
+  else
+    {
+      mu_message_t msg;
+      
+      if ((status = amd_get_message (mailbox, amd->msg_count - 1, &msg)) != 0 ||
+	  (status = mu_message_get_uid (msg, &uidnext)) != 0)
+	return status;
+      uidnext++;
+    }
+  status = _amd_prop_store_off (amd, _MU_AMD_PROP_UIDNEXT, uidnext);
+  if (status == 0)
+    status = _amd_prop_store_off (amd, _MU_AMD_PROP_UIDVALIDITY, uidvalidity);
+  return status;
 }
 
 static int
 amd_uidnext (mu_mailbox_t mailbox, size_t *puidnext)
 {
   struct _amd_data *amd = mailbox->data;
-  int status;
-  
-  if (!amd->next_uid)
-    return ENOSYS;
-  status = mu_mailbox_messages_count (mailbox, NULL);
+  int status = amd_initial_scan (amd);
   if (status != 0)
     return status;
-  /* If we did not start a scanning yet do it now.  */
-  if (amd->msg_count == 0)
-    {
-      status = _amd_scan0 (amd, 1, NULL, 0);
-      if (status != 0)
-	return status;
-    }
-   if (puidnext)
-     *puidnext = amd->next_uid (amd);
-  return 0;
+  return _amd_prop_fetch_size (amd, _MU_AMD_PROP_UIDNEXT, puidnext);
 }
 
 /* FIXME: effectively the same as mbox_cleanup */
@@ -1582,8 +1630,9 @@ msg_array_comp (const void *a, const void *b)
 void
 amd_sort (struct _amd_data *amd)
 {
-  qsort (amd->msg_array, amd->msg_count, sizeof (amd->msg_array[0]),
-	 msg_array_comp);
+  if (amd->msg_count)
+    qsort (amd->msg_array, amd->msg_count, sizeof (amd->msg_array[0]),
+	   msg_array_comp);
 }
 
 /* Scan given message and fill amd_message_t fields.
@@ -1606,7 +1655,7 @@ amd_scan_message (struct _amd_message *mhm)
   int amd_capa = amd->capabilities;
   
   /* Check if the message was modified after the last scan */
-  status = mhm->amd->cur_msg_file_name (mhm, &msg_name);
+  status = mhm->amd->cur_msg_file_name (mhm, 1, &msg_name);
   if (status)
     {
       mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
@@ -1648,22 +1697,9 @@ amd_scan_message (struct _amd_message *mhm)
 		  mu_c_strncasecmp (buf, "status:", 7) == 0)
 		{
 		  int deleted = mhm->attr_flags & MU_ATTRIBUTE_DELETED;
-		  mu_string_to_flags (buf, &mhm->attr_flags);
+		  mu_attribute_string_to_flags (buf, &mhm->attr_flags);
 		  mhm->attr_flags |= deleted;
 		}
-	      else if (!(amd_capa & MU_AMD_IMAPBASE) &&
-		       mu_c_strncasecmp (buf, "x-imapbase:", 11) == 0)
-		{
-		  if (_amd_prop_fetch_ulong (amd, _MU_AMD_PROP_UIDVALIDITY,
-					     NULL))
-		    {
-		      char *p;
-		      unsigned long uidval = strtoul (buf + 11, &p, 10);
-		      /* The next number is next uid. Ignored */
-		      _amd_prop_store_off (amd, _MU_AMD_PROP_UIDVALIDITY,
-					   uidval);
-		    }
-		}
 	    }
 	  else
 	    {
@@ -1817,7 +1853,7 @@ amd_message_stream_open (struct _amd_message *mhm)
   int status;
   int flags = 0;
 
-  status = amd->cur_msg_file_name (mhm, &filename);
+  status = amd->cur_msg_file_name (mhm, 1, &filename);
   if (status)
     {
       mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
@@ -2043,6 +2079,15 @@ amd_get_attr_flags (mu_attribute_t attr, int *pflags)
 
   if (mhm == NULL)
     return EINVAL;
+  if (!(mhm->amd->capabilities & MU_AMD_STATUS))
+    {
+      /* If AMD implementation doesn't handle status (attribute) bits, they
+	 must be retrieved from the Status: header.  To ensure that, the
+	 message must be scanned: */
+      int rc = amd_check_message (mhm);
+      if (rc)
+	return rc;
+    }
   if (pflags)
     *pflags = mhm->attr_flags;
   return 0;
@@ -2149,4 +2194,49 @@ amd_remove_dir (const char *name)
   return rc;
 }
 
+int
+amd_reset_uidvalidity (struct _amd_data *amd)
+{
+  struct timeval tv;
+  gettimeofday (&tv, NULL);
+  return _amd_prop_store_off (amd, _MU_AMD_PROP_UIDVALIDITY, tv.tv_sec);
+}
+
+int
+amd_update_uidnext (struct _amd_data *amd, size_t *newval)
+{
+  int rc;
+  size_t curval;
+  
+  rc = _amd_prop_fetch_size (amd, _MU_AMD_PROP_UIDNEXT, &curval);
+  if (rc == MU_ERR_NOENT)
+    curval = 1;
+  else if (rc)
+    return rc;
+  if (*newval < curval)
+    {
+      *newval = curval;
+      return 0;
+    }
+  return _amd_prop_store_off (amd, _MU_AMD_PROP_UIDNEXT, *newval);
+}
+
+int
+amd_alloc_uid (struct _amd_data *amd, size_t *newval)
+{
+  int rc;
+  size_t retval;
+  
+  rc = _amd_prop_fetch_size (amd, _MU_AMD_PROP_UIDNEXT, &retval);
+  if (rc == MU_ERR_NOENT)
+    retval = 1;
+  else if (rc)
+    return rc;
+  rc = _amd_prop_store_off (amd, _MU_AMD_PROP_UIDNEXT, retval + 1);
+  if (rc)
+    return rc;
+  *newval = retval;
+  return 0;
+}
+
 
