@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2011-2012, 2014-2017 Free Software Foundation, Inc.
+   Copyright (C) 2011-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -138,14 +138,16 @@ _mu_imap_folder_open (mu_folder_t folder, int flags)
   if (tls)
     {
       mu_stream_t tlsstream;
-	      
-      rc = mu_tls_client_stream_create (&tlsstream, transport, transport, 0);
+
+      rc = mu_tlsfd_stream_convert (&tlsstream, transport, NULL,
+				    MU_TLS_CLIENT);
       mu_stream_unref (transport);
       if (rc)
 	{
+	  if (rc == MU_ERR_TRANSPORT_SET)
+	    mu_stream_destroy (&tlsstream);
 	  mu_debug (MU_DEBCAT_FOLDER, MU_DEBUG_ERROR,
-		    (_("cannot create TLS stream: %s"),
-		     mu_strerror (rc)));
+		    ("cannot create TLS stream: %s", mu_strerror (rc)));
 	  return rc;
 	}
       transport = tlsstream;
@@ -153,7 +155,8 @@ _mu_imap_folder_open (mu_folder_t folder, int flags)
 #endif
 
   mu_imap_set_carrier (imap, transport);
-
+  mu_stream_unref (transport);
+  
   if (mu_debug_level_p (MU_DEBCAT_FOLDER, MU_DEBUG_PROT) ||
       mu_debug_level_p (MU_DEBCAT_MAILBOX, MU_DEBUG_PROT))
     mu_imap_trace (imap, MU_IMAP_TRACE_SET);
@@ -262,20 +265,97 @@ _enumerate_helper (void *item, void *data)
 }
 
 static int
-_mu_imap_folder_list (mu_folder_t folder, const char *ref, void *name,
-		      int flags, size_t max_level,
-		      mu_list_t flist,
-		      mu_folder_enumerate_fp efp, void *edp)
+_mu_imap_folder_separator (mu_folder_t folder, int *sep)
+{
+  mu_imap_t imap = folder->data;
+
+  if (imap->separator == 0)
+    {
+      struct mu_list_response *resp;
+      char const *path;
+      int rc;
+      mu_list_t list;
+      
+      if (mu_url_sget_path (folder->url, &path) || strcmp (path, "INBOX") == 0)
+	path = "";
+
+      rc = mu_imap_list_new (imap, path, "", &list);
+      if (rc)
+	return rc;
+      if (mu_list_head (list, (void**)&resp))
+	return ENOENT; /* FIXME: Better return code? */
+      imap->separator = resp->separator;
+      imap->prefix_len = strlen (path) + 1;
+      mu_list_destroy (&list);
+    }
+
+  if (sep)
+    *sep = imap->separator;
+
+  return 0;
+}
+
+static int
+_mu_imap_folder_pathname (mu_folder_t folder, char const *name, char **retname)
+{
+  char const *folder_path;
+  char *pathname;
+  
+  if (mu_url_sget_path (folder->url, &folder_path) ||
+      strcmp (folder_path, "INBOX") == 0)
+    {
+      pathname = strdup (name);
+    }
+  else
+    {
+      int sep;
+      size_t len;
+      int rc;
+	
+      rc = _mu_imap_folder_separator (folder, &sep);
+      if (rc)
+	return rc;
+      
+      len = strlen (folder_path) + (name ? strlen (name) : 0) + 2;
+      pathname = malloc (len);
+      if (pathname)
+	{
+	  char *p = mu_stpcpy (pathname, folder_path);
+	  *p++ = sep;
+	  if (name)
+	    mu_stpcpy (p, name);
+	}
+    }
+
+  if (!pathname)
+    return errno;
+
+  *retname = pathname;
+  
+  return 0;
+}
+
+/* NOTE: scn->records is ignored */
+static int
+_mu_imap_folder_list (mu_folder_t folder, struct mu_folder_scanner *scn)
 {
   mu_imap_t imap = folder->data;
   mu_list_t list;
-  int rc = mu_imap_list_new (imap, ref, name, &list);
-
+  int rc;
+  char *refname;
+      
+  rc = _mu_imap_folder_pathname (folder, scn->refname, &refname);
   if (rc)
     return rc;
 
-  if (max_level ||
-      (flags & MU_FOLDER_ATTRIBUTE_ALL) != MU_FOLDER_ATTRIBUTE_ALL)
+  rc = mu_imap_list_new (imap, refname, scn->pattern, &list);
+  free (refname);
+      
+  if (rc)
+    return rc;
+
+  if (scn->max_depth
+      || (scn->match_flags & MU_FOLDER_ATTRIBUTE_ALL) != MU_FOLDER_ATTRIBUTE_ALL)
     {
       /* Filter out the list, eliminating non-matching entries */
       mu_iterator_t itr;
@@ -293,25 +373,26 @@ _mu_imap_folder_list (mu_folder_t folder, const char *ref, void *name,
 	  struct mu_list_response *rp;
 
 	  mu_iterator_current (itr, (void**) &rp);
-	  if (!(rp->type & flags) || (max_level && rp->level > max_level))
+	  if (!(rp->type & scn->match_flags)
+	      || (scn->max_depth && rp->depth > scn->max_depth))
 	    mu_iterator_ctl (itr, mu_itrctl_delete, NULL);
 	}
       mu_iterator_destroy (&itr);
     }
 
-  if (efp)
+  if (scn->enumfun)
     {
       struct enumerate_closure clos;
 
       clos.folder = folder;
-      clos.fun = efp;
-      clos.data = edp;
+      clos.fun = scn->enumfun;
+      clos.data = scn->enumdata;
 
       rc = mu_list_foreach (list, _enumerate_helper, &clos);
     }
 
-  if (flist)
-    mu_list_append_list (flist, list);
+  if (scn->result)
+    mu_list_append_list (scn->result, list);
 
   mu_list_destroy (&list);
 
@@ -323,7 +404,15 @@ _mu_imap_folder_lsub (mu_folder_t folder, const char *ref, const char *name,
 		      mu_list_t flist)
 {
   mu_imap_t imap = folder->data;
-  return mu_imap_lsub (imap, ref, name, flist);
+  char *refpath;
+  int rc;
+  
+  if ((rc = _mu_imap_folder_pathname (folder, ref, &refpath)) == 0)
+    {
+      rc = mu_imap_lsub (imap, refpath, name, flist);
+      free (refpath);
+    }
+  return rc;
 }
 
 /* Subscribe to the named mailbox. */
@@ -331,14 +420,30 @@ static int
 _mu_imap_folder_subscribe (mu_folder_t folder, const char *name)
 {
   mu_imap_t imap = folder->data;
-  return mu_imap_subscribe (imap, name);
+  char *path;
+  int rc;
+  
+  if ((rc = _mu_imap_folder_pathname (folder, name, &path)) == 0)
+    {
+      rc = mu_imap_subscribe (imap, path);
+      free (path);
+    }
+  return rc;
 }
 /* Unsubscribe from the mailbox. */
 static int
 _mu_imap_folder_unsubscribe (mu_folder_t folder, const char *name)
 {
   mu_imap_t imap = folder->data;
-  return mu_imap_unsubscribe (imap, name);
+  char *path;
+  int rc;
+  
+  if ((rc = _mu_imap_folder_pathname (folder, name, &path)) == 0)
+    {
+      rc = mu_imap_unsubscribe (imap, path);
+      free (path);
+    }
+  return rc;
 }
 
 /* Remove a mailbox.  */
@@ -346,16 +451,36 @@ static int
 _mu_imap_folder_delete (mu_folder_t folder, const char *name)
 {
   mu_imap_t imap = folder->data;
-  return mu_imap_delete (imap, name);
+  char *path;
+  int rc;
+  
+  if ((rc = _mu_imap_folder_pathname (folder, name, &path)) == 0)
+    {
+      rc = mu_imap_delete (imap, path);
+      free (path);
+    }
+  return rc;
 }
 
-/* Rename OLDPATH to NEWPATH */
+/* Rename OLDNAME to NEWNAME */
 static int
-_mu_imap_folder_rename (mu_folder_t folder, const char *oldpath,
-			const char *newpath)
+_mu_imap_folder_rename (mu_folder_t folder, const char *oldname,
+			const char *newname)
 {
   mu_imap_t imap = folder->data;
-  return mu_imap_rename (imap, oldpath, newpath);
+  char *oldpath, *newpath;
+  int rc;
+
+  if ((rc = _mu_imap_folder_pathname (folder, oldname, &oldpath)) == 0)
+    {
+      if ((rc = _mu_imap_folder_pathname (folder, newname, &newpath)) == 0)
+	{
+	  rc = mu_imap_rename (imap, oldpath, newpath);
+	  free (newpath);
+	}
+      free (oldpath);
+    }
+  return rc;
 }
 
 typedef int (*auth_method_t) (mu_authority_t);
