@@ -126,11 +126,11 @@ mboxrd_open (mu_mailbox_t mailbox, int flags)
       && mailbox->locker == NULL
       && (flags & (MU_STREAM_WRITE | MU_STREAM_APPEND | MU_STREAM_CREAT)))
     {
-      rc = mu_locker_create (&mailbox->locker, dmp->name, 0);
+      rc = mu_locker_create_ext (&mailbox->locker, dmp->name, NULL);
       if (rc)
 	mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
 		  ("%s:%s (%s): %s",
-		   __func__, "mu_locker_create", dmp->name,
+		   __func__, "mu_locker_create_ext", dmp->name,
 		   mu_strerror (rc)));
     }
 
@@ -345,7 +345,7 @@ mboxrd_message_alloc_uid (struct mu_mboxrd_message *dmsg)
  * parameters, correspondingly.
  *
  * The header is stored in the first message.  To avoid rewriting entire
- * mailbox when one of the parameters chages, the values of <V> and <N>
+ * mailbox when one of the parameters changes, the values of <V> and <N>
  * are left-padded with spaces to the maximum width of their data types.
  *
  * Offset of the header in the mailbox and its length (without the
@@ -790,7 +790,8 @@ mboxrd_rescan_unlocked (mu_mailbox_t mailbox, mu_off_t offset)
 		 mu_strerror (rc)));
     }
   mu_stream_unref (stream);
-
+  free (buf);
+  
   if (force_init_uids)
     {
       dmp->uidvalidity = (unsigned long) time (NULL);
@@ -936,7 +937,8 @@ mboxrd_message_unseen (mu_mailbox_t mailbox, size_t *pmsgno)
 	}
     }
 
-  return MU_ERR_NOENT;
+  *pmsgno = 0;
+  return 0;
 }
 
 /* Initialize the mailbox UID subsystem. See the Notes above. */
@@ -1139,6 +1141,14 @@ mailbox_append_message (mu_mailbox_t mailbox, mu_message_t msg)
 		  if (rc == 0)
 		    {
 		      rc = mu_envelope_aget_date (env, &date);
+		      if (rc == MU_ERR_NOENT)
+			{
+			  date = strdup ("Thu Jan  1 00:00:00 1970");
+			  if (date)
+			    rc = 0;
+			  else
+			    rc = ENOMEM;
+			}
 		    }
 
 		  mu_envelope_destroy (&env, msg);
@@ -1183,7 +1193,7 @@ mailbox_append_message (mu_mailbox_t mailbox, mu_message_t msg)
       if (rc)
 	break;
 
-      rc = mu_filter_create (&flt, istr, "FROMRB",
+      rc = mu_filter_create (&flt, istr, "FROMRD",
 			     MU_FILTER_ENCODE, MU_STREAM_READ);
       mu_stream_destroy (&istr);
       rc = mu_stream_copy_nl (mailbox->stream, flt, 0, NULL);
@@ -1193,11 +1203,12 @@ mailbox_append_message (mu_mailbox_t mailbox, mu_message_t msg)
 
   if (rc)
     {
+      int rc1;
       mu_stream_destroy (&istr);
-      rc = mu_stream_truncate (mailbox->stream, size);
-      if (rc)
+      rc1 = mu_stream_truncate (mailbox->stream, size);
+      if (rc1)
 	mu_error (_("cannot truncate stream after failed append: %s"),
-		  mu_stream_strerror (mailbox->stream, rc));
+		  mu_stream_strerror (mailbox->stream, rc1));
       return rc;
     }
 
@@ -1995,6 +2006,7 @@ mboxrd_detect (char const *name)
 		}
 	    }
 	  free (buf);
+	  mu_stream_destroy (&str);
 	}
     }
   return res;
