@@ -140,17 +140,20 @@ pop_open (mu_mailbox_t mbox, int flags)
 #ifdef WITH_TLS
   if (mpd->pops)
     {
-      mu_stream_t newstr;
-      
-      status = mu_tls_client_stream_create (&newstr, stream, stream, 0);
+      mu_stream_t tlsstream;
+
+      status = mu_tlsfd_stream_convert (&tlsstream, stream, NULL,
+					MU_TLS_CLIENT);
       mu_stream_unref (stream);
       if (status)
 	{
-	  mu_error ("pop_open: mu_tls_client_stream_create: %s",
-		    mu_strerror (status));
+	  if (status == MU_ERR_TRANSPORT_SET)
+	    mu_stream_destroy (&tlsstream);
+	  mu_debug (MU_DEBCAT_FOLDER, MU_DEBUG_ERROR,
+		    ("cannot create TLS stream: %s", mu_strerror (status)));
 	  return status;
 	}
-      stream = newstr;
+      stream = tlsstream;
     }
 #endif /* WITH_TLS */
 
@@ -164,7 +167,8 @@ pop_open (mu_mailbox_t mbox, int flags)
       return status;
     }
   mu_pop3_set_carrier (mpd->pop3, stream);
-
+  mu_stream_unref (stream);
+  
   if (mu_debug_level_p (MU_DEBCAT_MAILBOX, MU_DEBUG_PROT))
     mu_pop3_trace (mpd->pop3, MU_POP3_TRACE_SET);
   if (mu_debug_level_p (MU_DEBCAT_MAILBOX, MU_DEBUG_TRACE6))
@@ -760,14 +764,14 @@ pop_create_header (struct _pop3_message *mpm)
 /* Attributes */
 
 /* There is no POP3 command to return message attributes, therefore we
-   have to recurse to reading the "Status:" header.  Unfortunately, some
-   servers remove it when you dowload a message, and in this case a message
+   have to recourse to reading the "Status:" header.  Unfortunately, some
+   servers remove it when you download a message, and in this case a message
    will always look like new even if you already read it.  There is also
    no way to set an attribute on remote mailbox via the POP server and
    many server once you do a RETR (and in some cases a TOP) will mark the
    message as read (Status: RO).  Even worse, some servers may be configured
    to delete after the RETR, and some go as much as deleting after the TOP,
-   since technicaly you can download a message via TOP without RETR'eiving
+   since technically you can download a message via TOP without RETR'eiving
    it.  */
 static int
 pop_get_attribute (mu_attribute_t attr, int *pflags)
