@@ -62,7 +62,7 @@ static const struct mail_command_entry mail_command_table[] = {
     mail_file, file_compl },
   { "folders",	"folders",	"folders",			0,
     mail_folders, no_compl },
-  { "fo",	"followup",	"fo[llowup] [msglist]",		EF_SEND,
+  { "fo",	"followup",	"fo[llowup] [message]",		EF_SEND,
     mail_followup, msglist_compl },
   { "f",	"from",		"f[rom] [msglist]",		0,
     mail_from, msglist_compl },
@@ -84,6 +84,8 @@ static const struct mail_command_entry mail_command_table[] = {
     mail_list, no_compl },
   { "m",	"mail",		"m[ail] [address...]",		EF_SEND,
     mail_send, alias_compl },
+  { "M",        "Mail",         "M[ail] [address...]",		EF_SEND,
+    mail_send, alias_compl },
   { "mb",	"mbox",		"mb[ox] [msglist]",		0,
     mail_mbox, msglist_compl },
   { "n",	"next",		"n[ext] [message]",		0,
@@ -108,9 +110,9 @@ static const struct mail_command_entry mail_command_table[] = {
     mail_reply, msglist_compl },
   { "R",	"Respond",	"R[espond] [msglist]",		EF_SEND,
     mail_reply, msglist_compl },
-  { "r",	"reply",	"r[eply] [msglist]",		EF_SEND,
+  { "r",	"reply",	"r[eply] [message]",		EF_SEND,
     mail_reply, msglist_compl },
-  { "r",	"respond",	"r[espond] [msglist]",		EF_SEND,
+  { "r",	"respond",	"r[espond] [message]",		EF_SEND,
     mail_reply, msglist_compl },
   { "ret",	"retain",	"ret[ain] [header-field]",	0,
     mail_retain, no_compl },
@@ -150,6 +152,8 @@ static const struct mail_command_entry mail_command_table[] = {
     mail_print, msglist_compl },
   { "una",	"unalias",	"una[lias] [alias]...",		0,
     mail_unalias, NULL },
+  { "U",        "unread",       "unread [msglist]",             0,
+    mail_unread, msglist_compl },
   { "u",	"undelete",	"u[ndelete] [msglist]",		0,
     mail_undelete, msglist_compl },
   { "unf",      "unfold",       "unf[old] [header-field]",      0,
