@@ -302,7 +302,7 @@ static struct mu_cli_setup cli = {
   options,
   sieve_cfg_param,
   N_("GNU sieve -- a mail filtering tool."),
-  "SCRIPT",
+  N_("SCRIPT"),
   NULL,
   N_("Sieve-specific debug levels:\n\
 \n\
@@ -429,7 +429,7 @@ sieve_mailbox (mu_sieve_machine_t mach)
       int e;
       
       /* A message won't be marked deleted unless the script executed
-         succesfully on it, so we always do an expunge, it will delete
+         successfully on it, so we always do an expunge, it will delete
          any messages that were marked DELETED even if execution failed
          on a later message. */
       if ((e = mu_mailbox_expunge (mbox)) != 0)
